provider "alicloud" {
  region = var.region
}

// Zone
data "alicloud_zones" "default" {
  available_disk_category     = "cloud_efficiency"
  available_resource_creation = "Instance"

}
// Instance specifications：1 vCPU and 2 GiB memory.
data "alicloud_instance_types" "default" {
  availability_zone    = data.alicloud_zones.default.zones.0.id
  spot_strategy        = "SpotWithPriceLimit"
  cpu_core_count       = 1
  memory_size          = 2
}
// Image
data "alicloud_images" "default" {
  most_recent   = true
  instance_type = data.alicloud_instance_types.default.instance_types[0].id
  owners        = "system"
  name_regex    = "^aliyun_3_x64"
}

resource "alicloud_vpc" "default" {
  description = "havip-demo-vpc"
  cidr_block  = "192.168.0.0/16"
  vpc_name    = "havip-demo-vpc"
}

resource "alicloud_vswitch" "default" {
  vswitch_name = "havip-demo"
  cidr_block   = "192.168.0.0/24"
  vpc_id       = alicloud_vpc.default.id
  zone_id      = data.alicloud_zones.default.zones.0.id
}

resource "alicloud_havip" "default" {
  vswitch_id  = alicloud_vswitch.default.id
  ip_address  = "192.168.0.24"
  description = "havip-demo"
  ha_vip_name = "havip-demo"
}

resource "alicloud_havip_attachment" "havip_attachement_1" {
  havip_id    = alicloud_havip.default.id
  instance_id = alicloud_instance.instance1.id
  force       = true
}

resource "alicloud_havip_attachment" "havip_attachement_2" {
  havip_id    = alicloud_havip.default.id
  instance_id = alicloud_instance.instance2.id
  force       = true
}

resource "alicloud_security_group" "default" {
  name   = "havip-demo"
  vpc_id = alicloud_vpc.default.id
}

resource "alicloud_security_group_rule" "allow_all_tcp" {
  type              = "ingress"
  ip_protocol       = "tcp"
  nic_type          = "intranet"
  policy            = "accept"
  port_range        = "1/65535"
  priority          = 1
  security_group_id = alicloud_security_group.default.id
  cidr_ip           = "0.0.0.0/0"
}

resource "alicloud_instance" "instance1" {
  security_groups         = alicloud_security_group.default.*.id
  instance_type           = data.alicloud_instance_types.default.instance_types[0].id
  image_id                = data.alicloud_images.default.images[0].id
  system_disk_category    = "cloud_efficiency"
  system_disk_name        = "havip-demo"
  system_disk_description = "havip-demo"
  instance_name           = "havip-demo-ecs1"
  vswitch_id              = alicloud_vswitch.default.id
  user_data               = file("scripts/ecs1-init.sh")
  private_ip              = "192.168.0.25"
  instance_charge_type    = "PostPaid"
  spot_strategy           = "SpotWithPriceLimit"
}

resource "alicloud_instance" "instance2" {
  security_groups         = alicloud_security_group.default.*.id
  instance_type           = data.alicloud_instance_types.default.instance_types[0].id
  image_id                = data.alicloud_images.default.images[0].id
  system_disk_category    = "cloud_efficiency"
  system_disk_name        = "havip-demo"
  system_disk_description = "havip-demo"
  instance_name           = "havip-demo-ecs2"
  vswitch_id              = alicloud_vswitch.default.id
  user_data               = file("scripts/ecs2-init.sh")
  private_ip              = "192.168.0.26"
  instance_charge_type    = "PostPaid"
  spot_strategy           = "SpotWithPriceLimit"
}

resource "alicloud_instance" "instance3" {
  availability_zone       = data.alicloud_zones.default.zones.0.id
  instance_name           = "havip-demo-ecs3"
  security_groups         = alicloud_security_group.default.*.id
  vswitch_id              = alicloud_vswitch.default.id
  instance_type           = data.alicloud_instance_types.default.instance_types[0].id
  system_disk_category    = "cloud_efficiency"
  system_disk_name        = "havip-demo"
  system_disk_description = "havip-demo"
  image_id                = data.alicloud_images.default.images[0].id
  private_ip              = "192.168.0.27"
  instance_charge_type    = "PostPaid"
  spot_strategy           = "SpotWithPriceLimit"
}

resource "time_sleep" "wait_60_seconds" {
  depends_on = [
    alicloud_instance.instance1,
    alicloud_instance.instance2
  ]

  create_duration = "60s"
}

