# 一键部署三个 ECS 实例 + HA VIP 实现双机主备高可用

## 1. 运行
### 1.1 准备工作
Cloud Shell 内置了 terraform 以及临时身份凭证，建议将本工程上传到 CloudShell 中执行。 
> 如果在本地运行，需要确保本地有[安装 Terraform](https://developer.hashicorp.com/terraform/downloads)。
### 1.2 创建资源
在 CloudShell 打开本工程根目录，执行命令来创建资源：
> 在 Cloud Shell 中可以直接执行 terraform apply，背后会默认使用登录 Cloud Shell 身份对应的临时凭证。   
```bash
terraform init
terraform apply
```

如果在本地运行，请执行
```bash
export ALICLOUD_ACCESS_KEY="<your_access_key_id>"
export ALICLOUD_SECRET_KEY="<your_access_key_secret>"
terraform init
terraform apply
```
### 1.3 验证
等待片刻后，终端会输出 ha vip 控制台的地、 ha vip 的私网 IP 以及 ECS 1 、 ECS 3 的实例 ID 。
您可以登陆 ECS 控制台，登录 ECS 3 实例，连接 ha vip ，默认会显示 “ECS 1”。
停止 ECS 1 实例，稍等几秒中后，然后再次连接 ha vip ，将会显示 “ECS 2”。

### 1.4 销毁资源
在本工程根目录，执行以下命令，即可销毁本次测试所创建的所有资源，避免继续产生费用。
```bash
terraform destroy -var="access_key_id=xxx" -var="access_key_secret=xxx"
```
> xxx 请替换为真实的 access_key_id 和 access_key_secret
> 在 Cloud Shell 中可以直接执行 terraform apply 不添加 -var 参数，背后会默认使用登录 Cloud Shell 身份对应的临时凭证。


## 2. 参考文档
- [什么是高可用虚拟 IP](https://help.aliyun.com/document_detail/110065.html)
- [HaVip结合keepalived实现主备双机高可用
](https://help.aliyun.com/document_detail/184485.html)
- [如何使用 Terraform](https://help.aliyun.com/document_detail/95825.html)
                   
**--------------------------------------ENGLISH----VERSION------------------------------------------------------**
                   
# One-click deployment of three ECS instances + HA VIP to achieve dual-machine active-standby high availability
## 1. Running
### 1.1 Preparation
Before running, make sure that Terraform is installed locally. [Install Terraform](https://developer.hashicorp.com/terraform/downloads)

Cloud Shell has built-in Terraform and temporary identity credentials, and you can also upload this project to Cloud Shell for execution.
### 1.2 Creating Resources
Open the root directory of this project locally and run the following command to create resources:            
```bash
terraform init
terraform apply -var="access_key_id=xxx" -var="access_key_secret=xxx"
```              
> Replace xxx with the real access_key_id and access_key_secret. In Cloud Shell, you can directly execute `terraform apply` without adding the `-var `parameter, and the corresponding temporary credentials for the login identity will be used by default.
### 1.3 Verification
After waiting for a while, the terminal will output the address of the HAVIP console, the private IP address of HAVIP, the instance ID of ecs 1 instance and ecs 3 instance. which will display "ECS 1" by default. 
You can log in to the ECS console, access the ecs 3 instance, and connect to the HAVIP. It will display "ECS 1" by default. 
You can stop the ecs 1 instance, wait a few seconds, and then connect to the HAVIP again. It will display "ECS 2".
### 1.4 Destroying Resources
In the root directory of this project, execute the following command to destroy all resources created in this test to avoid incurring further costs:
```bash
terraform destroy -var="access_key_id=xxx" -var="access_key_secret=xxx"
``` 
> Replace xxx with the real access_key_id and access_key_secret. In Cloud Shell, you can directly execute `terraform apply` without adding the `-var` parameter, and the corresponding temporary credentials for the login identity will be used by default behind the scenes.
## 2. References
- [Overview of HAVIPs](https://www.alibabacloud.com/help/en/virtual-private-cloud/latest/havips-overview)
- [Implement high availability by using HAVIPs and keepalived](https://www.alibabacloud.com/help/en/virtual-private-cloud/latest/implement-high-availability-by-using-havips-and-keepalived)
- [Install and configure Terraform](https://www.alibabacloud.com/help/en/terraform/latest/install-and-configure-terraform-in-the-local-pc)
